@@ -11,7 +11,7 @@ use URPM::Resolve;
 use URPM::Signature;
 
 our @ISA = qw(DynaLoader);
-our $VERSION = '4.27';
+our $VERSION = '4.9';
 
 URPM->bootstrap($VERSION);
 
@@ -32,7 +32,7 @@ sub set_nofatal {
 
 sub packages_providing {
     my ($urpm, $name) = @_;
-    grep { $_ } map { $urpm->{depslist}[$_] } sort { $a <=> $b } keys %{$urpm->{provides}{$name} || {}};
+    grep { $_ } map { $urpm->{depslist}[$_] } keys %{$urpm->{provides}{$name} || {}};
 }
 
 sub packages_obsoleting {
@@ -292,7 +292,7 @@ URPM - Manipulate RPM files and headers
 =head1 DESCRIPTION
 
 The URPM module allows you to manipulate RPM files, RPM header files and
-hdlist files and manage them in memory. It is notably used by the L<urpmi>
+hdlist files and manage them in memory. It is notably used by the C<urpmi>
 utility. It provides four classes : C<URPM>, C<URPM::DB>, C<URPM::Package>,
 and C<URPM::Transaction>.
 
@@ -303,28 +303,15 @@ and C<URPM::Transaction>.
 =item URPM->new()
 
 The constructor creates a new, empty URPM object. It's a blessed hash that
-contains three fields:
-
-=over
-
-=item *
+contains two fields:
 
 B<depslist> is an arrayref containing the list of depending packages (which are
 C<URPM::Package> objects).
 
-=item *
-
-B<obsoletes> is an hashref containing as keys the list of property names
-obsoleted by the URPM object.
-
-=item *
-
 B<provides> is an hashref containing as keys the list of property names
 provided by the URPM object. The associated value is true if the property is
 versioned.
 
-=back
-
 If the constructor is called with the arguments C<< nofatal => 1 >>, various
 fatal error messages are suppressed (file not found in parse_hdlist() and
 parse_synthesis()).
@@ -347,11 +334,6 @@ where epoch, version and release are RPM-style version numbers.
 This method gets the B<depslist> and the B<provides> from a synthesis file
 and adds them to the URPM object.
 
-Callback signature is callback(C<URPM>, C<URPM::Package>).
-
-The return value is a two-element array containing the first and the last id
-parsed.
-
 =item $urpm->parse_hdlist($file, %options)
 
 This method loads rpm informations from rpm headers contained in an hdlist
@@ -361,8 +343,6 @@ file and adds them to the URPM object. Allowed options are
     callback => sub { ... }
     keep_all_tags => 0 / 1
 
-Callback signature is callback(C<URPM>, C<URPM::Package>).
-
 The return value is a two-element array containing the first and the last id
 parsed.
 
@@ -378,8 +358,6 @@ and adds them to the URPM object. Allowed options are
 If C<keep_all_tags> isn't specified, URPM will drop all memory-consuming tags
 (notably changelogs, filelists, scriptlets).
 
-Callback signature is callback(URPM::Package).
-
 =item $urpm->packages_providing($name)
 
 Returns a list of C<URPM::Package> providing <$name>
@@ -412,8 +390,6 @@ $names is a reference to an array, holding the acceptable values of the said
 tag for the searched variables.
 Then, $callback is called for each matching package in the depslist.
 
-Callback signature is callback(URPM::Package).
-
 This is used when faking a URPM::DB: $urpm can be used as-a $db
 
 =item $urpm->traverse_tag_find($tag,$name,$callback)
@@ -422,8 +398,6 @@ Quite similar to C<traverse_tag>, but stops when $callback returns true.
 
 (also note that only one $name is handled)
 
-Callback signature is callback(URPM::Package).
-
 This is used when faking a URPM::DB: $urpm can be used as-a $db
 
 =item URPM::verify_rpm($file, %options)
@@ -483,8 +457,6 @@ Rebuilds the RPM database (like C<rpm --rebuilddb>). $prefix defaults to C<"">.
 Executes the specified callback (a code reference) for each package
 in the DB, passing a C<URPM::Package> object as argument the callback.
 
-Returns the number of packages seen (all).
-
 =item $db->traverse_tag($tag,$names,$callback)
 
 $tag may be one of C<name>, C<whatprovides>, C<whatrequires>, C<whatconflicts>,
@@ -493,24 +465,16 @@ $names is a reference to an array, holding the acceptable values of the said
 tag for the searched variables.
 Then, $callback is called for each matching package in the DB.
 
-Callback signature is callback(URPM::Package).
-
-Returns the number of packages seen (all those that matched provided names).
-
 =item $db->traverse_tag_find($tag,$name,$callback)
 
 Quite similar to C<traverse_tag>, but stops when $callback returns true.
 
 (also note that only one $name is handled)
 
-Callback signature is callback(URPM::Package).
-
-Returns whether callback returned true once.
-
-=item $db->create_transaction()
+=item $db->create_transaction($prefix)
 
 Creates and returns a new transaction (an C<URPM::Transaction> object) on the
-specified DB.
+specified DB. For $prefix, cf L<open>.
 
 =back
 
@@ -568,8 +532,6 @@ This is only used when faking a URPM::DB: $urpm can be used as-a $db
 
 =item $package->epoch()
 
-=item $package->EVR()
-
 =item $package->excludearchs()
 
 =item $package->exclusivearchs()
@@ -787,12 +749,10 @@ list context, returns an array of problems (an empty array indicates success).
 Runs the transaction.
 
 $data is an arbitrary user-provided piece of data to be passed to callbacks.
-It's usually the $urpm object.
 
 Recognized options are:
 
     callback_close  => sub { ... }
-    callback_error  => sub { ... }
     callback_inst   => sub { ... }
     callback_open   => sub { ... }
     callback_trans  => sub { ... }
@@ -804,24 +764,12 @@ Recognized options are:
     nosize => 0 / 1
     noscripts => 0 / 1
     oldpackage => 0 / 1
+    repackage => 0 / 1
     test => 0 / 1
     translate_message => 1
 
 They roughly correspond to command-line options to rpm(1).
 
-'callback_open' signature is (C<$data>, C<$cb_type>, C<$pkg_id>). It _must_ return a file handler for the asked package.
-
-'callback_close' signature is (C<$data>, C<$cb_type>, C<$pkg_id>). It is called just before URPM close the fd for the installed package.
-
-C<$cb_type> is one of 'open' or 'close'.
-
-Other Callbacks signature is callback(C<$data>, C<$cb_type>, C<$pkg_id>, C<$subtype>, C<$amout>, C<$total>)
-
-C<$cb_type> is one of 'error', 'inst', 'trans' or 'uninst'. C<$subtype> can be 'start', 'progress' or 'stop'.
-For 'error', it can be 'cpio', 'script' or 'unpack'.
-
-The purpose of those callbacks is to report progress (the two last parameters (C<$amount> & C<$total>) enable to compute progress percentage).
-
 =item $trans->traverse($callback)
 
 Executes the specified callback (a code reference) for each package in the
@@ -861,12 +809,6 @@ Delete a macro.
 
 Destroys macros.
 
-=back
-
-=head2 Misc other functions
-
-=over
-
 =item setVerbosity($level)
 
 Sets rpm verbosity level. $level is an integer between 2 (RPMMESS_CRIT) and 7
@@ -945,9 +887,7 @@ Copyright 2002, 2003, 2004, 2005 MandrakeSoft SA
 
 Copyright 2005, 2006, 2007, 2008 Mandriva SA
 
-Copyright 2011, 2012, 2013 Mageia
-
-FranE<ccedil>ois Pons (original author), Rafael Garcia-Suarez, Pixel, Thierry Vignaud <tv@mageia.org> (current maintainer)
+FranE<ccedil>ois Pons (original author), Rafael Garcia-Suarez, Pixel <pixel@mandriva.com> (current maintainer)
 
 This library is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
