@@ -11,11 +11,6 @@ use Config;
 # perl_checker: require URPM
 
 #- a few functions from MDK::Common copied here:
-sub any(&@) {
-    my $f = shift;
-    $f->($_) and return 1 foreach @_;
-    0;
-}
 sub listlength {
     my (@l) = @_;
     scalar @l;
@@ -68,7 +63,7 @@ sub removed_or_obsoleted_packages {
 #- (nb: see also find_required_package())
 #-
 #- side-effects: none
-sub find_candidate_packages {
+sub find_candidate_packages_ {
     my ($urpm, $id_prop, $o_rejected) = @_;
     my @packages;
 
@@ -95,11 +90,24 @@ sub find_candidate_packages {
     @packages;
 }
 
+#- deprecated, use find_candidate_packages_() directly
+#-
+#- side-effects: none
+sub find_candidate_packages {
+    my ($urpm, $id_prop, $o_rejected) = @_;
+
+    my %packages;
+    foreach (find_candidate_packages_($urpm, $id_prop, $o_rejected)) {
+	push @{$packages{$_->name}}, $_;
+    }
+    \%packages;
+}
+
 #- returns the "arch" of package $n in rpm db
 sub get_installed_arch {
     my ($db, $n) = @_;
     my $arch;
-    $db->traverse_tag_find('name', $n, sub { $arch = $_[0]->arch; 1 });
+    $db->traverse_tag('name', [ $n ], sub { $arch = $_[0]->arch });
     $arch;
 }
 
@@ -117,12 +125,11 @@ my %installed_arch;
 #- side-effects: none (but uses a cache)
 sub strict_arch_check_installed {
     my ($db, $pkg) = @_;
-    my $arch = $pkg->arch;
-    if ($arch ne 'src' && $arch ne 'noarch') {
+    if ($pkg->arch ne 'src' && $pkg->arch ne 'noarch') {
 	my $n = $pkg->name;
 	defined $installed_arch{$n} or $installed_arch{$n} = get_installed_arch($db, $n);
 	if ($installed_arch{$n} && $installed_arch{$n} ne 'noarch') {
-	    $arch eq $installed_arch{$n} or return;
+	    $pkg->arch eq $installed_arch{$n} or return;
 	}
     }
     1;
@@ -134,11 +141,9 @@ sub strict_arch_check_installed {
 #- side-effects: none
 sub strict_arch_check {
     my ($installed_pkg, $pkg) = @_;
-    my $arch = $pkg->arch;
-    if ($arch ne 'src' && $arch ne 'noarch') {
-	my $inst_arch = $installed_pkg->arch;
-	if ($inst_arch ne 'noarch') {
-	    $arch eq $inst_arch or return;
+    if ($pkg->arch ne 'src' && $pkg->arch ne 'noarch') {
+	if ($installed_pkg->arch ne 'noarch') {
+	    $pkg->arch eq $installed_pkg->arch or return;
 	}
     }
     1;
@@ -151,7 +156,7 @@ sub is_package_installed {
     my ($db, $pkg) = @_;
 
     my $found;
-    $db->traverse_tag_find('name', $pkg->name, sub {
+    $db->traverse_tag('name', [ $pkg->name ], sub {
 	my ($p) = @_;
 	$found ||= $p->fullname eq $pkg->fullname;
     });
@@ -184,9 +189,12 @@ sub provided_version_that_overlaps {
     $version;
 }
 
+#- deprecated function, use find_required_package()
+sub find_chosen_packages { &find_required_package }
+
 #- find the package (or packages) to install matching $id_prop
 #- returns (list ref of matches, list ref of preferred matches)
-#- (see also find_candidate_packages())
+#- (see also find_candidate_packages_())
 #-
 #- side-effects: flag_install, flag_upgrade (and strict_arch_check_installed cache)
 sub find_required_package {
@@ -217,11 +225,12 @@ sub find_required_package {
 	    $may_add_to_packages->($pkg);
 	} elsif (my $name = property2name($_)) {
 	    my $property = $_;
-	    foreach my $pkg (packages_providing($urpm, $name)) {
+	    foreach (sort { $a <=> $b } keys %{$urpm->{provides}{$name} || {}}) {
+		my $pkg = $urpm->{depslist}[$_];
 		$pkg->is_arch_compat or next;
 		$pkg->flag_skip || $state->{rejected}{$pkg->fullname} and next;
 		#- check if at least one provide of the package overlaps the property
-		if (!$urpm->{provides}{$name}{$pkg->id} || $pkg->provides_overlap($property)) {
+		if (!$urpm->{provides}{$name}{$_} || $pkg->provides_overlap($property)) {
 		    #- determine if this package is better than a possibly previously chosen package.
 		    $pkg->flag_selected || exists $state->{selected}{$pkg->id} and return [$pkg];
 		    !$strict_arch || strict_arch_check_installed($db, $pkg) or next;
@@ -350,7 +359,7 @@ sub _find_required_package__kmod {
     $choices->[0]->name =~ /^dkms-|-kernel-\d\./ or return;
 
     grep {
-	if (my ($version, $flavor, $release) = $_->name =~ /(?:.*)-kernel-(\d\..*)-(.*)-(.*)/) {
+	if (my ($_name, $version, $flavor, $release) = $_->name =~ /(.*)-kernel-(\d\..*)-(.*)-(.*)/) {
 	    my $kernel = "kernel-$flavor-$version-$release";
 	    _is_selected_or_installed($urpm, $db, $kernel);
 	} elsif ($_->name =~ /^dkms-/) {
@@ -379,7 +388,7 @@ sub _score_for_locales {
 	  } else {
 	      0; # bad locale
 	  }
-    } elsif (any { /locales-en/ } @r) {
+    } elsif (grep { /locales-en/ } @r) {
 	2; # 
     } else {
 	1;
@@ -572,7 +581,7 @@ sub backtrack_selected {
 
 	    #- search for all possible packages, first is to try the selection, then if it is
 	    #- impossible, backtrack the origin.
-	    my @packages = find_candidate_packages($urpm, $dep->{required});
+	    my @packages = find_candidate_packages_($urpm, $dep->{required});
 
 	    foreach (@packages) {
 		    #- avoid dead loop.
@@ -873,20 +882,13 @@ sub resolve_requested {
     my @selected = resolve_requested__no_suggests($urpm, $db, $state, $requested, %options);
 
     if (!$options{no_suggests}) {
-        @selected = resolve_requested_suggests($urpm, $db, $state, \@selected, %options);
-    }
-    @selected;
-}
-
-sub resolve_requested_suggests {
-    my ($urpm, $db, $state, $selected, %options) = @_;
-	my @todo = @$selected;
+	my @todo = @selected;
 	while (@todo) {
 	    my $pkg = shift @todo;
 	    my %suggests = map { $_ => 1 } $pkg->suggests or next;
 
 	    #- do not install a package that has already been suggested
-	    $db->traverse_tag_find('name', $pkg->name, sub {
+	    $db->traverse_tag('name', [ $pkg->name ], sub {
 		my ($p) = @_;
 		delete $suggests{$_} foreach $p->suggests;
 	    });
@@ -906,11 +908,11 @@ sub resolve_requested_suggests {
 	    my %new_requested = map { $_ => undef } keys %suggests;
 	    my @new_selected = resolve_requested__no_suggests_($urpm, $db, $state, \%new_requested, %options);
 	    $state->{selected}{$_->id}{suggested} = 1 foreach @new_selected;
-	    push @$selected, @new_selected;
+	    push @selected, @new_selected;
 	    push @todo, @new_selected;
 	}
-
-    @$selected;
+    }
+    @selected;
 }
 
 #- Resolve dependencies of requested packages; keep resolution state to
@@ -935,7 +937,7 @@ sub resolve_requested__no_suggests {
 
     foreach (keys %$requested) {
 	#- keep track of requested packages by propating the flag.
-	foreach (find_candidate_packages($urpm, $_)) {
+	foreach (find_candidate_packages_($urpm, $_)) {
 	    $_->set_flag_requested;
 	}
     }
@@ -966,7 +968,6 @@ sub resolve_requested__no_suggests_ {
     #- then examine provides that will be removed (which need to be satisfied by another
     #- package present or by a new package to upgrade), then requires not satisfied and
     #- finally conflicts that will force a new upgrade or a remove.
-    my $count = 1;
     do {
 	while (my $dep = shift @properties) {
 	    #- we need to avoid selecting packages if the source has been disabled.
@@ -983,7 +984,7 @@ sub resolve_requested__no_suggests_ {
 	    } else {
 		_set_flag_installed_and_upgrade_if_no_newer($db, $pkg);
 
-		if ($pkg->flag_installed && !$pkg->flag_upgrade && !$urpm->{options}{downgrade}) {
+		if ($pkg->flag_installed && !$pkg->flag_upgrade) {
 		    _no_more_recent_installed_and_providing($urpm, $db, $state, $pkg, $dep->{required}) or next;
 		}
 	    }
@@ -1054,13 +1055,7 @@ sub resolve_requested__no_suggests_ {
 	} elsif (my $dep = shift @choices) {
 	    push @properties, $dep;
 	}
-
-	# safety:
-	if ($count++ > 50000) {
-	    die("detecting looping forever while trying to resolve dependancies.\n"
-		. "Aborting... Try again with '-vv --debug' options");
-	}
-    } while (@diff_provides || @properties || @choices);
+    } while @diff_provides || @properties || @choices;
 
     #- return what has been selected by this call (not all selected hash which may be not empty
     #- previously. avoid returning rejected packages which weren't selectable.
@@ -1076,7 +1071,7 @@ sub resolve_requested__no_suggests_ {
 sub _handle_conflicts_with_selected {
     my ($urpm, $db, $state, $pkg, $dep, $properties, $diff_provides, %options) = @_;
     foreach ($pkg->conflicts) {
-	if (my $n = property2name($_)) {
+	if (my ($n, $_o, $_v) = property2name_op_version($_)) {
 	    foreach my $p ($urpm->packages_providing($n)) {
 		$pkg == $p and next;
 		$p->provides_overlap($_) or next;
@@ -1188,9 +1183,9 @@ sub _unselect_package_deprecated_by_property {
 	    if ($o && $comparison > 0) {
 		#- installed package is newer
 		#- remove this package from the list of packages to install,
-		#- unless urpmi was invoked with --allow-force 
-		#- (in which case rpm could be invoked with --oldpackage)
-		if (!$urpm->{options}{'allow-force'} && !$urpm->{options}{downgrade}) {
+		#- unless urpmi was invoked with --allow-force (in which
+		#- case rpm could be invoked with --oldpackage)
+		if (!$urpm->{options}{'allow-force'}) {
 		    #- since the originally requested packages (or other
 		    #- non-installed ones) could be unselected by the following
 		    #- operation, remember them, to warn the user
@@ -1269,7 +1264,7 @@ sub _handle_diff_provides {
 	#- try if upgrading the package will be satisfying all the requires...
 	#- there is no need to avoid promoting epoch as the package examined is not
 	#- already installed.
-	my @packages = find_candidate_packages($urpm, $p->name, $state->{rejected});
+	my @packages = find_candidate_packages_($urpm, $p->name, $state->{rejected});
 	@packages = 
 	  grep { ($_->name eq $p->name ? $p->compare_pkg($_) < 0 :
 		    $_->obsoletes_overlap($p->name . " == " . $p->epoch . ":" . $p->version . "-" . $p->release))
@@ -1289,7 +1284,7 @@ sub _handle_diff_provides {
 	    #- there exists enough packages that provided the unsatisfied requires.
 	    my @best;
 	    foreach (@unsatisfied) {
-		my @packages = find_candidate_packages($urpm, $_, $state->{rejected});
+		my @packages = find_candidate_packages_($urpm, $_, $state->{rejected});
 		if (@packages = grep { $_->fullname ne $p->fullname } @packages) {
 		    push @best, join('|', map { $_->id } @packages);
 		}
@@ -1329,7 +1324,7 @@ sub _handle_conflict {
     #- whether a newer version will be ok, else ask to remove the old.
     my $need_deps = $p->name . " > " . ($p->epoch ? $p->epoch . ":" : "") .
       $p->version . "-" . $p->release;
-    my @packages = grep { $_->name eq $p->name } find_candidate_packages($urpm, $need_deps, $state->{rejected});
+    my @packages = grep { $_->name eq $p->name } find_candidate_packages_($urpm, $need_deps, $state->{rejected});
     @packages = grep { ! $_->provides_overlap($property) } @packages;
 
     if (!@packages) {
@@ -1617,6 +1612,7 @@ sub compute_flag {
 #-   + those of compute_flag (flag_skip, flag_disable_obsolete)
 sub compute_flags {
     my ($urpm, $val, %options) = @_;
+    if (ref $val eq 'HASH') { $val = [ keys %$val ] } #- compatibility with urpmi <= 4.5-13mdk
     my @regex;
 
     #- unless a regular expression is given, search in provides
@@ -1717,7 +1713,7 @@ sub request_packages_to_upgrade {
 		$pkg = undef;
 	    }
 	}
-	if ($pkg && $options{idlist} && !any { $pkg->id == $_ } @{$options{idlist}}) {
+	if ($pkg && $options{idlist} && !grep { $pkg->id == $_ } @{$options{idlist}}) {
 		$urpm->{debug_URPM}("not auto-selecting " . $pkg->fullname . "because it's not in search medias") if $urpm->{debug_URPM};
 		$pkg = undef;
 	} 
@@ -1807,7 +1803,7 @@ sub sort_graph {
 		# don't care
 	    } elsif (exists $added{$p_id}) {
 		# already done
-	    } elsif (any { $_ == $p_id } @ids) {
+	    } elsif (grep { $_ == $p_id } @ids) {
 		my $begin = 1;
 		my @l = grep { $begin &&= $_ != $p_id } @ids;
 		$loop_ahead = 1;
@@ -2010,7 +2006,7 @@ sub build_transaction_set {
 
     if ($state->{orphans_to_remove}) {
 	my @l = map { scalar $_->fullname } @{$state->{orphans_to_remove}};
-	push @{$state->{transaction}}, { remove  => \@l } if @l;
+	push @{$state->{transaction}}, { remove  => \@l };
     }
 
     $state->{transaction};
